# -*- coding: utf-8 -*-
from django.apps import AppConfig
import urllib, requests, json
from timetable.models import Course

class SearchConfig(AppConfig):
    name = 'curso'
    
class SearchOb(object):
	"""docstring for SearchOb"""
	def __init__(self, keyword="", school=None, uri=None):
		from pymongo import MongoClient
		self.client = MongoClient(uri)
		self.db = self.client['timetable']
		self.SrchCollect = self.db['CourseSearch']
		self.TitleAbbre = self.db['TitleAbbre']

		self.keyword = keyword.split()
		self.school = school
		self.result = tuple()

	def getResult(self):
		self.doSearch()
		return self.result

	def doSearch(self):
		if len(self.keyword) == 1:
			self.keyword = self.keyword[0]
			self.result = self.KEMSearch(self.keyword)
		else:
			self.result = self.TCsearch()

	def KEMSearch(self, kw):
		from functools import reduce 

		cursor = self.SrchCollect.find({'key':kw}, {'value':1, '_id':False}).limit(1)
		if cursor.count() > 0:
			# Key Exist
			return list(cursor)[0]['value']
		else:
			try:
				kcm = json.loads(requests.get('http://api.udic.cs.nchu.edu.tw/api/kcm/?keyword={}&lang=cht&num=200'.format(urllib.parse.quote(kw))).text)
				kem = json.loads(requests.get('http://api.udic.cs.nchu.edu.tw/api/kem/?keyword={}&lang=cht&num=200'.format(urllib.parse.quote(kw))).text)


				for i in reduce(lambda x, y: x + y, zip(kcm, kem)):
					cursor = self.SrchCollect.find({'key':i[0]}, {self.school:1, '_id':False}).limit(1)
					if cursor.count() > 0:
						# Key Exist
						value = list(cursor)[0][self.school]
						self.SrchCollect.update({'key':kw}, {'$set': {self.school:value}}, upsert=True)
						return value

				return []
			except Exception as e:
				print(e)
				return []

	def TCsearch(self):
		cursor1 = self.KEMSearch(self.keyword[0])
		cursor2 = self.KEMSearch(self.keyword[1])
		intersection = set(cursor1).intersection(cursor2)
		if intersection == []:
			if cursor1:
				return cursor1
			elif cursor2:
				return cursor2
			else:
				return []

		for i in self.keyword[2:]:
			cursor2 = self.KEMSearch(i)
			intersection = intersection.intersection(cursor2)
		return list(intersection)

	def incWeight(self, fullTitle):
		self.TitleAbbre.update({'key':self.keyword[0]}, {'$inc':{'value.{}'.format(fullTitle):1}}, upsert=True)


from djangoApiDec.djangoApiDec import queryString_required
from django.http import JsonResponse
from .apps import SearchOb

# Create your views here.
@queryString_required(['keyword', 'school'])
def search(request):
	keyword = request.GET['keyword']
	school = request.GET['school']
	sob = SearchOb(keyword, school)
				
	return JsonResponse(sob.getResult(), safe=False)

@queryString_required(['keyword', 'fullTitle'])
def incWeight(request):
	keyword = request.GET['keyword']
	fullTitle = request.GET['fullTitle']
	sob = SearchOb(keyword)
	sob.incWeight(fullTitle)
	return JsonResponse({"receive Weight success":1}, safe=False)


# -*- coding: utf-8 -*-
from django.conf.urls import url
from curso import views
urlpatterns = [
	url(r'^get/search/$', views.search, name='search'),
	url(r'^post/incWeight/$', views.incWeight, name='incWeight'),
]


from django.test import TestCase
from django.test import Client
from django.core.urlresolvers import reverse

# Create your tests here.
class CursoTestCase(TestCase):
	def setUp(self):
		self.client = Client()

	def test_clist(self):
		response = self.client.get(reverse('curso:search')+"?keyword=文化+臺灣&school=NUTC")
		self.assertEqual(response.json(), ["D16432"])

	def test_incWeight(self):
		response = self.client.get(reverse('curso:incWeight')+'?keyword=發心&fullTitle=發展心理學')
		self.assertEqual(response.json(), {"receive Weight success": 1})